@@ -5,7 +5,15 @@
 
 =head1 NAME
 
-B<Params::Clean> (Parse A Routine Allowing Modest Syntax--Casually List Explicit Arg Names): Process @_ as positional/named/flag/list/typed arguments
+=over 1
+
+=item Params::Clean
+
+I<Parse A Routine Allowing Modest Syntax: Casually List Explicit Arg Names>
+
+Process @_ as positional/named/flag/list/typed arguments
+
+=back
 
 =cut
 
@@ -52,14 +60,12 @@ Instead of starting your sub with C<my ($x, $y, $z) = @_;>
   my ($either_end, $tint) = args [0, -1], [Colour, Color];
 
 
-=head1 VERSION
-
-Version 0.9.4 (December 2007)
-
-This version introduces the PARSE keyword.
+=for html
+<STYLE>BODY { font-family: Optima, "Lucida Sans", Tahoma, Verdana, "Bitstream Vera Sans"; } </STYLE>
 
 =cut
 
+	#...add a pinch of CSS because CPAN.org inexplicably uses Arial(!)  =P
 
 
 
@@ -70,41 +76,44 @@ This version introduces the PARSE keyword.
 	#===========================================================================
 
 	package Params::Clean;
-	use version; our $VERSION = qv"0.9.4";
+
+	die __PACKAGE__.": Cannot find version number!\n" unless <<EOP =~/^\s*Version\s+([\d\.]+)\s+/im;
+
+=head1 VERSION
+
+Version 0.9 (July 2007): Initial release
+
+=cut
+
+EOP
+	our $VERSION=$1;	# Pull v# out of POD so that we need only specify it in one place
 	
 	use 5.6.0;																			# Because we use "our", etc.
 	use strict; use warnings; no warnings qw(uninitialized);							# Be good little disciplinarians (but not too good)
+	use warnings::register;																# Handle our own warnings
 	use Devel::Caller::Perl 'called_args';		                                      	# for stealing our caller's @_
 	
 	
-	our (@keywords, @KEYWORDS);					                                    	# We need to declare these and then init them with BEGIN so they're ready for the "use UID"
-	BEGIN { our @keywords=qw/POSN NAME FLAG REST TYPE PARSE/; }	                       	# UID keywords
-	BEGIN { our @KEYWORDS=(@keywords, "LIST", "args"); }		                       	# all keywords (LIST handled specially)
+	use base "Exporter";
+	our @EXPORT=qw/args POSN NAME FLAG REST TYPE LIST/;									# Symbols to be exported by default
+
 	
-	use UID @keywords;			# Set up some lexicals that won't be available anywhere else, so exporting refs to them will act as unique identifiers
 	
-	our %Warn;																			# categories of warning levels by caller: e.g. $Warn{main}{missing_start}=fatal
-	BEGIN {
-		$Warn{undef}={			               	# default warning levels
-					invalid_opts=>"warn", 		# illegal warning or keyword options used
-					funny_arglist=>"ignore",	# asked to PARSE something that's not an ARRAY, HASH, or CODE
-					missing_start=>"ignore", 	# LIST cannot find specified starting key
-					missing_end=>"warn", 		# LIST cannot find specified ending key
-					invalid_list=>"warn", 		# tried to use a FLAG or LIST, etc, as endpoint to a LIST
-					invalid_type=>"warn", 		# tried to use an illegal TYPE definition
-					nonint_name=>"warn",  		# non-integral key will be used as a name
-					orphaned_type=>"warn", 		# TYPE not followed by a definition
-					misplaced_rest=>"warn", 	# REST used before last parameter
-					misplaced_parse=>"die", 	# PARSE used after first parameter
-					 };
-		}
-	# now create constants with all our exception-type names (handy, and helps catch typos!)	
-	BEGIN { no strict 'refs'; for my $s (keys %{$Warn{undef}}) {*{$s}=sub {return $s, @_ if wantarray; warn "ERROR: attempt to use args after '$s' which is in scalar context (perhaps you need a comma after '$s'?)" if @_; return $s};} }	# stolen from UID.pm
 	
+	#===========================================================================
+	#
+	# 	IDs & DECLARATIONS
+	#
+	#===========================================================================
+	
+	# Set up some lexicals that won't be available anywhere else,
+	#  so exporting refs to them will act as unique identifiers
+	
+	use UID qw/POSN NAME FLAG TYPE REST/;		# LIST handled specially
 	
 	our $CaseSensitive=0;																# By default, we match match names case-insensitively
 	our $Debug=0;																		# Whether to show debugging messages (0 level=none)
-	sub same($$);	sub insame($@);	sub typewriter($$); sub warning;					# predeclare!
+	sub same($$);	sub insame($@);	sub typewriter($$);									# predeclare!
 	sub un {grep !$_[$_], 0..@_-1;}														# pull out all the keys that work out to false (used with @used!)
 	sub array { map ref($_) eq "ARRAY"?@$_:$_, (@_) }									# Normalise a list by expanding array-refs
 	sub comma { "[".join(", ", array @_)."]" } 											# Format array(ref) into "[a, b, c]"
@@ -115,53 +124,12 @@ This version introduces the PARSE keyword.
 	#	Pass each thing you want cleaned as a separate arg
 	{ 
 		return unless $Debug>=shift;													# do nothing unless our debugging level is high enough
-		my $i; my %ID=reverse(POSN=>POSN, FLAG=>FLAG, NAME=>NAME, TYPE=>TYPE, REST=>REST);		# lookup hash for our special IDs
-		warn join " ", map $ID{$_}?"|$ID{$_}|":ref eq"ARRAY"?"[".(join " ", map $ID{$_}?"|$ID{$_}|":$_, (@$_))."]":ref eq "HASH"?"{".(join "", map {$i++%2?"$_; ":"$_=>"} %$_)."}":"$_", (@_), "\n"
+		my %ID=reverse(POSN=>POSN, FLAG=>FLAG, NAME=>NAME, TYPE=>TYPE, REST=>REST);		# lookup hash for our special IDs
+		warn join " ", map $ID{$_}?"|$ID{$_}|":ref($_)eq"ARRAY"?"[".(join " ", map $ID{$_}?"|$ID{$_}|":$_, (@$_))."]":"$_", (@_), "\n"
 	}
 	
 	
 
-	#===========================================================================
-	#
-	# 	STARTUP
-	#
-	#===========================================================================
-	
-	sub import
-	# Handle module options: renaming exported UIDs and setting desired warnings
-	#
-	# RENAMING: pass a keyword ID followed by the new name (LIST=>"PLIST") -- setting to undef means don't export it at all
-	# WARNINGS: warn=>"type", or die=>"type" or fatal=>"type", or ignore=>"type"
-	{
-		my $me=shift; 							# our package name
-		my @opts, my $i; push @opts, [$_[$i++]=>$_[$i++]] while $i<@_;	# pair up the options (we would use a hash, but we want to preserve order, and anyway we could have the same key repeated)
-		my %EXPORT=map {$_=>$_} @KEYWORDS;		# keywords to be exported (normally all @KEYWORDS) in convenient hash format
-		my $keys=join "|", @KEYWORDS;			# for regex to test for any of our keywords
-		my $caller=(caller)[0];					# caller's package
-		
-		
-		# Set up warning/fatal/ignoral categories
-		$Warn{$caller}={%{$Warn{undef}}};					      		# start by setting up default warning levels
-		for (grep $opts[$_][0]=~/^(warn|die|fatal|ignore)$/, 0..$#opts)	# grep through the key-halves of each opt for exception-levels
-		{
-			my $opt=delete $opts[$_];
-			warning(invalid_opts qq[WARNING: Ignoring attempt to set unrecogised warning category "$opt->[1]"]) and next unless exists $Warn{$caller}{$opt->[1]};	# complain if trying to set an invalid category
-			$Warn{$caller}{$opt->[1]}=$opt->[0];		# set level for this caller and remove opts as we handle them
-		}
-		
-		
-		# Look for our keywords: pairs that start with a keyword substitute the new name instead
-		$EXPORT{$opts[$_][0]}=$opts[$_][1] and delete $opts[$_] for grep $opts[$_][0]=~/^($keys)$/, grep exists $opts[$_], 0..$#opts; # look for our keywords and remove opts as we deal with them
-		no strict 'refs';		                                                        # so we can manually "export" the subs to the caller's namespace
-		*{$caller."::".$EXPORT{$_}}=\&{$_} for grep defined $EXPORT{$_}, keys %EXPORT;	# skipping undefs
-		
-		
-		# If there are any opts left, we don't know what to do with them
-		warning invalid_opts "WARNING: Ignoring unrecognised options [".join(", ", map "$opts[$_][0]=>$opts[$_][1]", grep exists $opts[$_], 0..$#opts)."]" if @opts;
-	}
-	
-	
-	
 	#===========================================================================
 	#
 	# 	LISTs
@@ -210,12 +178,7 @@ This version introduces the PARSE keyword.
 		#------------------------------------------------------
 		
 		my @sig=@_;															# The signature specifying how to parse the caller's args
-		
-		# Get args to be parsed
-		if (same $sig[0], PARSE)		    		                    	# then specially passed in the list to parse
-		{	shift @sig; @_=preparse(shift @sig);	}  			         	# drop first arg(=PARAM) and grab the second(=arrayref)
-		else 		                        		                    	# we use [the caller's] @_ by default
-		{	@_=called_args(0);	}											# get the @_ args passed in to the original sub (=our caller)
+		@_=called_args(0);													# Get the @_ args passed in to the original sub (=our caller)
 		
 		my $n;																# Counter for which parameter we're processing
 		my $type;															# holder for the ID of the arg-type currently being processed
@@ -245,18 +208,11 @@ This version introduces the PARSE keyword.
 	my $typesub;	
 		for my $param (@sig)
 		{
-			warning misplaced_rest "WARNING: attempt to use REST before last parameter" and $rest++ if $rest==1;		# complain if REST flag is set and we're still looping (i.e. not done with the sig) [increment and check only when ==1 so the warning doesn't spam us every time through the loop!]
-			
-			warning misplaced_parse "ERROR: encountered PARSE after beginning of parameter list" if same $param, PARSE;	# complain if PARSE wasn't the first parameter (would've been dealt with above)
+			warnings::warn "WARNING: attempt to use REST before last parameter" and $rest++ if $rest==1;	# complain if REST flag is set and we're still looping (i.e. not done with the sig) [increment and check only when ==1 so the warning doesn't spam us every time through the loop!]
 			
 			#Switch type whenever we hit one of our identifiers
 			
-			if ($type==PARSE)																# We found a PARSE keyword last pass through (which was an error, of course)
-			{
-				warning misplaced_parse "\tIgnoring misplaced PARSE values"; 				# but too late to do anything with them
-				undef $type;																# reset for next arg
-			}
-			elsif ($typesub)																# previous item was a TYPE type, so look for the sub
+			if ($typesub)																	# previous item was a TYPE type, so look for the sub
 			{
 				$param=[TYPE, $param];														# put our TYPE=>sub into an array-ref so we can deal with it as a single unit below
 				$typesub=0;
@@ -267,7 +223,7 @@ This version introduces the PARSE keyword.
 			{
 				$typesub=1;																	# set flag so next pass we can grab the type-sub
 			}
-			elsif (insame $param => POSN, NAME, FLAG, PARSE)								# we've hit one of our types
+			elsif (insame $param => POSN, NAME, FLAG)										# we've hit one of our types
 			{
 				$type=$param;  																# Switch current type-holder to that type
 				debug 2, "\t", $type, "type";
@@ -321,7 +277,7 @@ This version introduces the PARSE keyword.
 						$err.=" (probably already used up by another param!)" if insame $param->{start}->[0], @_;	# more helpful message -- if starting keyword really is in the arg list, then we most likely can't find it because it already got used somewhere else
 					}
 					
-					warning missing_start $err; 
+					warnings::warn $err; 
 					
 					$results[$n++]=[];	push @number, undef; 								# add an empty result since we could find it properly
 					next;
@@ -392,7 +348,7 @@ This version introduces the PARSE keyword.
 								$err.=" (probably already used up by another param!)" if insame $param->{end}->[0], @_;	# more helpful message -- if ending keyword really is in the arg list, then we most likely can't find it because it already got used somewhere else
 							}
 							
-							warning missing_end $err; 
+							warnings::warn $err; 
 							$end=$args-1 unless defined $end; 	#to grab all until end... or should we skip this because of the error: "next;" ??
 						}
 						elsif (!$param->{incl})
@@ -483,7 +439,7 @@ This version introduces the PARSE keyword.
 							else															# not a type of TYPE that we recognise!
 							{
 								debug 2, "ERROR! Invalid TYPE!!!\t#$n\tKey[$i]:", $key, "\tType:", $kind, "\tArg[$a]:", $_[$a];
-								warning invalid_type "WARNING: attempt to use invalid TYPE";
+								warnings::warn "WARNING: attempt to use invalid TYPE";
 							}
 							
 							if ($match)
@@ -594,8 +550,7 @@ This version introduces the PARSE keyword.
 		{
 			return POSN if $param==int($param);											# numeric and an int
 			###Maybe warn if some kind of ref? not an object?? Hm....
-			##perhaps use "$param"<0, etc., since a stringified int will still numify to an int...
-			warning nonint_name if "WARNING: non-integral number $param will be interpreted as a named parameter";
+			warnings::warnif "WARNING: non-integral number $param will be interpreted as a named parameter";
 		}
 		
 		# Not an int, so assume named
@@ -604,57 +559,6 @@ This version introduces the PARSE keyword.
 	
 	
 	
-	#===========================================================================
-	#
-	# 	WARNINGS
-	#
-	#===========================================================================
-	
-	sub warning
-	# Display a warning message, or die, or do nothing, according to our error levels
-	{
-		my $category=shift;					# error category, as controlled by %Warn
-		my $level=1;						# start one level up (our caller)
-		my @caller=(caller $level);	    	# to find out whose settings to use; 
-		@caller=(caller ++$level) while $caller[0] eq __PACKAGE__;		# keep moving a level up until we go beyond our own package
-		
-		my $w=$Warn{$caller[0]}{$category};
-		
-		return if $w eq "ignore";
-		warn "@_ at $caller[1] line $caller[4]\n";
-		die "\t(Fatal exception category: $category)\n" if $w eq "die" or $w eq "fatal";
-	}
-	
-	
-	
-	#===========================================================================
-	#
-	# 	PREPARSE LIST of ARGS
-	#
-	#===========================================================================
-	
-	sub preparse	
-	# Get the list of args to be parsed, passed in via a PARSE keyword
-	{	
-		my $args=shift;							# we pass in a single value
-		my $ref=ref $args || "value";
-		
-		# normally, the list should be passed in as an array-ref
-		return @$args if $ref eq "ARRAY";
-		
-		# but might be a hashref, we just expand as a list
-		return %$args if $ref eq "HASH";
-		
-		# of it we've got a coderef, call it and return the results
-		return &$args if $ref eq "CODE";
-		
-		# anything else, just assume it's the only arg and return it!
-		warning funny_arglist "WARNING: suspicious arg-list given to PARSE (a single unrecognised $ref)";	
-		return $args;
-	}
-	
-	
-	
 	#===========================================================================
 	#
 	# 	PARSE PARAMS
@@ -701,7 +605,7 @@ This version introduces the PARSE keyword.
 			}
 		}
 		
-		warning orphaned_type "WARNING: Orphaned TYPE" if $typesub; 	         		# we found a TYPE but no type-sub was following it!
+		warnings::warn "WARNING: Orphaned TYPE" if $typesub; 	                       	# we found a TYPE but no type-sub was following it!
 		
 		return \@keys, \@types;
 	}	
@@ -806,18 +710,6 @@ Trying to mix named and positional params in the middle of your args, though, is
 
 
 
-=head2 Specifying the argument list
-
-By default, C<args> parses C<@_> to get the list of arguments.  You can override this with the C<PARSE> keyword, 
-which takes a single value to be used for the args list.  For example, C<args PARSE \@_, ...> would explicitly get its arguments from C<@_>.
-You can use any array-ref, or a hash-ref which will be flattened and treated as a plain list, or a code-ref which will be called and
-the results used as the argument list.  
-Anything else will be used as a (single) argument value.
-
-The C<PARSE> keyword and its value must come immediately after C<args>; putting other parameters before it will raise an error.
-
-
-
 =head2 POSN/NAME/FLAG identifiers
 
 You can also explicitly identify the kind of parameter using the keywords C<POSN> or C<NAME>.
@@ -1142,29 +1034,6 @@ Again, once C<Pipe> is found, it does not matter whether the values identified b
 even if that means the list consists of nothing but the starting point.)
 
 
-=head2 Care and C<Usage> of your module
-
-You can simply C<use Params::Clean>, or you can supply some extra options to control warnings and exported names.
-The options are a series of keys and values (so they must be correctly paired).
-
-To change the name under which a keyword will be exported into your namespace, give its default name followed by
-the name you wish to use for it in your calling module, e.g. if you already have a C<LIST> function, you can rename
-C<Params::Clean>'s C<LIST> by including an option like C<< LIST=>PLIST >>.
-
-You can also control how C<Params::Clean> will handle various kinds of errors.  Most exceptions simply emit a warning
-message and try to continue.  You can set the level for recognised categories to "warn" to display a message; 
-to "die" or "fatal" to display the message and die; or to "ignore" to do nothing.
-Give the level of error-handling followed by the category name, e.g. C<< die=>missing_start >>.
-See L<Diagnostics|"DIAGNOSTICS"> for the names of each category, and the default level.
-
-Example:
-
-	use Params::Clean  LIST=>"PLIST", NAME=>"Key",  fatal=>"misplaced_rest";
-
-C<Params::Clean> will issue a warning for any unrecognised options that it encounters.  (You can C<< ignore=>invalid_opts >>,
-but of course that will affect only subsequent options, not any that came before it.)
-
-
 
 =head1 UIDs
 
@@ -1201,103 +1070,56 @@ The same considerations apply as for exporting any other subroutine
 -- allow the user control over what gets exported to avoid conflicts from different modules trying to export UIDs of the same name.
 
 C<Params::Clean> exports UIDs for its identifiers (C<NAME, POSN, FLAG, TYPE, REST, LIST>) so that you can use them with the C<args> function in your subroutines.
-(They can be renamed for importing into your namespace: see L<"Care and Usage of your module">).
 
 
 
 
 =head1 DIAGNOSTICS
 
-The list below includes the category of each exception, so that you can control how C<Params::Clean> handles that type
-of exception, e.g. C<< warn=>foo >> means that any "foo" errors will issue a warning by default.
-(See L<"Care and Usage of your module">).
-
-
 =over 1
 
-=item I<WARNING: Ignoring attempt to set unrecogised warning category>
-
-=item I<WARNING: Ignoring unrecognised options>
-
-B<C<< warn=>invalid_opts >>>
-
-An option (pair) given in the C<use> statement is invalid, misspelled, or otherwise not recognised by C<Params::Clean>.
-The unknown option will be skipped over.
-
-
-=item I<ERROR: encountered PARSE after beginning of parameter list>
-
-B<C<< fatal=>misplaced_parse >>>
-
-When explcitly giving a list of arguments to parse, the C<PARSE> keyword must be the first thing passed to C<args>.
-By default, C<Params::Clean> will die when it finds a C<PARSE> command out of place; 
-if you set it to C<ignore> or C<warn>, the value passed in via C<PARSE> will be ignored
-(and if you have set C<< warn=>misplaced_parse >>, you will get a "B<Ignoring misplaced PARSE values>" message).
-
-
-=item I<WARNING: suspicious arg-list given to PARSE (a single unrecognised value)>
-
-B<C<< ignore=>funny_arglist >>>
-
-The value you pass in for an argument list using C<PARSE> should be an arrayref, or a hashref, or a coderef.
-Anything else will trigger this warning, if you turn it on.
-
-
-=item I<WARNING: attempt to use REST before last parameter>
-
-B<C<< warn=>misplaced_rest >>>
+=item WARNING: attempt to use REST before last parameter
 
 The C<REST> keyword was not the last item passed to C<args>.  The leftover values are always returned after everything else,
 so C<REST> should appear last to avoid confusion.
 
 
-=item I<WHOA: can't use other LISTs inside a LIST!  Ignoring starting >[orI< ending>]I< param key: $key>
+=item WHOA: can't use other LISTs inside a LIST!  Ignoring starting I<[or> endingI<]> param key: $key
 
-=item I<WHOA: can't use FLAGs or TYPEs inside a LIST!  Ignoring starting >[orI< ending>]I< param key: $key>
-
-B<C<< warn=>invalid_list >>>
+=item WHOA: can't use FLAGs or TYPEs inside a LIST!  Ignoring starting I<[or> endingI<]> param key: $key
 
 A C<LIST> can take only named or positional parameters as the starting (or ending) point.  
-Something like C<< LIST [FLAG Foo] <=> [TYPE \&foo] >> will trigger a warning for either the starting or ending point (or both).
+Something like C<< LIST [FLAG Foo] <=> [TYPE \&foo] >> will trigger a warning for either the starting or ending point (or both),
 An invalid starting point means nothing will be returned for the list (C<undef>); 
 an invalid ending point means that only the starting key will be returned; no other args will be collected.
 
 
-=item I<ERROR: couldn't find beginning of LIST starting with '$key'>
+=item ERROR: couldn't find beginning of LIST starting with '$key'
 
-=item I<ERROR: couldn't find ending of LIST from $start to $end>
-
-B<C<< ignore=>missing_start >>>
-
-B<C<< warn=>missing_end >>>
+=item ERROR: couldn't find ending of LIST from $start to $end
 
 The starting or ending parameter specified for a LIST could not be found. 
-If the given parameter does appear somewhere in C<@_>, the message will also say, I<"(probably already used up by another param!)">
-(meaning a previously-collected arg already marked that parameter as "used" -- see L<"Using up arguments">).
+If the given parameter does appear somewhere in C<@_>, the message will also say, B<"(probably already used up by another param!)">
+(meaning a previously-collected arg already marked that parameter as "used" 
+-- see L<"Using up arguments">).
 If the starting point cannot be found, then nothing (C<undef>) is returned for the list (surprisingly enough).
 If the ending point cannot be found, then everything else (not already collected) until the end of C<@_> will be grabbed by the list.
 To deliberately allow a list to run off the end of C<@_>, make C<-1> (one of) the ending keys, or else do not specify an ending point at all.
 
 
-=item I<WARNING: attempt to use invalid TYPE>
-
-B<C<< warn=>invalid_type >>>
+=item WARNING: attempt to use invalid TYPE
 
 C<TYPE> parameters must be the name of a class (a C<ref> value), or a code-ref that can check each arg.
 Trying to use anything else as a C<TYPE> (e.g. a plain number or string) will result in this error.
 
 
-=item I<WARNING: non-integral number $param will be interpreted as a named parameter>
-
-B<C<< warn=>nonint_name >>>
+=item WARNING: non-integral number $param will be interpreted as a named parameter
 
 A number that's not an integer was found as a parameter key.  Since positional params must be integers,
 the value will be interpreted as a C<NAME>d parameter.  To avoid the error, explicitly mark the key using the C<NAME> keyword.
 
 
-=item I<WARNING: Orphaned TYPE>
-
-B<C<< warn=>orphaned_type >>>
+=item WARNING: Orphaned TYPE
 
 A C<TYPE> keyword was encountered without a following string or coderef, e.g., C<args 1,2, [TYPE];>.
 
@@ -1351,13 +1173,7 @@ even if it does add slightly to the clutter.  Judicious use of C<< => >> to quot
 C<LIST>s cannot identify starting (or ending) points by C<TYPE>.  They probably should be able to.
 
 
-Additional or more helpful diagnostics would be nice.
-
-
-Sometimes, trying to read C<@_> automatically seems not to work.  If this happens, the simple workaround is to explicitly
-specify C<PARSE \@_> as the first thing passed to C<args>.  
-(And if you know what makes Devel::Caller::Perl's C<called_args> function sometimes unable to read C<@_>, please let me know!)
-
+Additional or more helpful diagnostics would be nice, and users should have more control over them.
 
 To paraphrase L<Damian Conway|Getopt::Declare>: 
 It shouldn't take hundreds and hundreds of lines to explain a package that was designed for intuitive ease of use!
@@ -1374,7 +1190,7 @@ This module requires L<UID.pm|UID> and L<Devel::Caller::Perl>.
 
 =head1 METADATA
 
-Copyright 2007-2008 David Green, C<< <plato at cpan.org> >>.  
+Copyright 2007 David Green, C<< <plato at cpan.org> >>.  
 
 This module is free software; you may redistribute it or modify it under the same terms as Perl itself. See L<perlartistic>. 
 
